import express, { Request } from "express";
import { connect } from "mongoose";
import { ProductService } from "./productService";
import { Product } from "./product";
import dotenv from "dotenv";

const swaggerUi = require("swagger-ui-express");
const swaggerDocument = require("../swagger.json");

dotenv.config();

const app = express();
const port = process.env.PORT || 3000;

const productService = new ProductService();

app.use(express.json());

app.get(
  "/api/products",
  async (req: Request<{}, Product[], {}, { n: number }>, res) => {
    const count = req.query.n || 20;

    res.send(await productService.getN(count));
  }
);

app.get(
  "/api/products/:sku",
  async (req: Request<{ sku: string }, Product | null>, res) => {
    const product = await productService.getBySku(req.params.sku);

    if (product) {
      res.send(product);
    } else {
      res.sendStatus(404);
    }
  }
);

app.post("/api/products", async (req: Request<{}, {}, Product>, res) => {
  const product = req.body;

  await productService.create(product);

  res.location(`/api/products/${product.Sku}`);
  res.status(201);
  res.send();
});

app.put("/api/products", async (req: Request<{}, {}, Product>, res) => {
  const product = req.body;

  await productService.update(product);

  res.location(`/api/products/${product.Sku}`);
  res.status(201);
  res.send();
});

app.delete("/api/products/:sku", async (req: Request<{ sku: string }>, res) => {
  await productService.delete(req.params.sku);

  res.send();
});

app.use("/api-docs", swaggerUi.serve, swaggerUi.setup(swaggerDocument));

app.listen(port, async () => {
  if (!process.env.COSMOSDB_CONNECTIONSTRING) {
    console.error(
      "Ensure you create a `COSMOSDB_CONNECTIONSTRING` environment variable containing your CosmosDB connection string."
    );
    process.exit(-1);
  }
  await connect(process.env.COSMOSDB_CONNECTIONSTRING, {
    useNewUrlParser: true,
    useUnifiedTopology: true,
    dbName: process.env.DATABASE_NAME,
  });
  console.log(`Quickstart is running at http://localhost:${port}`);
});
